// Generated by CoffeeScript 1.10.0
(function() {
  var EventEmitter, HGCommandServer, OutputLine, _, defaults, spawn,
    extend = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty,
    slice = [].slice;

  EventEmitter = require("events").EventEmitter;

  spawn = require("child_process").spawn;

  _ = require("lodash");

  OutputLine = require("./common/OutputLine");

  defaults = {
    hgOpts: ['--config', 'ui.interactive=True', '--config', 'ui.merge=internal:fail', 'serve', '--cmdserver', 'pipe']
  };


  /*
  An HGCommandServer spawns the hg command server and handles communication between it and node.
  
  Piped output is then 'emitted' via the `EventEmitter` subclass as `output`, `result` or `error` events.
   */

  HGCommandServer = (function(superClass) {
    extend(HGCommandServer, superClass);

    function HGCommandServer(config) {
      if (config == null) {
        config = {};
      }
      this.config = _.defaults(config, defaults);
      this.server = this.starting = false;
    }


    /*
    	Start the command server at a specified directory (path must already be an hg repository)
     */

    HGCommandServer.prototype.start = function(path, done) {
      return this._spawnCommandServer(path, done);
    };


    /*
    	Stop the current command server process from running
     */

    HGCommandServer.prototype.stop = function() {
      if (!this.server) {
        return;
      }
      this.server.stdout.removeAllListeners("data");
      this.server.stderr.removeAllListeners("data");
      return this.server.stdin.end();
    };


    /*
    	Commands with arguments (like runcommand 8 log -l 5)
    	More info at [runcommand](http://mercurial.selenic.com/wiki/CommandServer#runcommand)
     */

    HGCommandServer.prototype.issueCommand = function() {
      var args, cmd;
      cmd = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
      return this._serverSend(cmd, args);
    };


    /*
    	Spawn the hg cmdserver as a child process
     */

    HGCommandServer.prototype._spawnCommandServer = function(path, done) {
      _.bindAll(this, "_handleServerExit", "_handleServerData");
      this.starting = true;
      this.server = this._makeHgProcess(path);
      this.server.stdout.once("data", (function(_this) {
        return function(data) {
          var capability, i, len, line, ref, ref1;
          _this.starting = false;
          line = OutputLine.FromBuffer(data, 0, void 0);
          ref = _this._parseCapabilitiesAndEncoding(line.body), _this.capabilities = ref.capabilities, _this.encoding = ref.encoding;
          ref1 = _this.capabilities;
          for (i = 0, len = ref1.length; i < len; i++) {
            capability = ref1[i];
            _this[capability] = function() {
              var args;
              args = 1 <= arguments.length ? slice.call(arguments, 0) : [];
              return _this.issueCommand.apply(_this, [capability].concat(args));
            };
          }
          _this.server.stdout.on("data", _this._handleServerData);
          return done(null, _this.capabilities, _this.encoding);
        };
      })(this));
      this.server.stderr.on("data", (function(_this) {
        return function(data) {
          if (_this.starting) {
            return done(new Error(data));
          }
          return _this._handleServerError(data);
        };
      })(this));
      return this.server.on("exit", this._handleServerExit);
    };


    /*
    	Create the child process (exposed for unit testing mostly)
     */

    HGCommandServer.prototype._makeHgProcess = function(path) {
      var processEnv, spawnOpts;
      processEnv = _.extend({
        "HGENCODING": "UTF-8"
      }, process.env);
      spawnOpts = {
        env: processEnv,
        cwd: path || process.cwd()
      };
      return spawn('hg', this.config.hgOpts, spawnOpts);
    };


    /*
    	Parse the capabilities and encoding when the cmd server starts up
     */

    HGCommandServer.prototype._parseCapabilitiesAndEncoding = function(data) {
      var capabilities, encoding, fields;
      fields = {};
      data.split('\n').forEach(function(line) {
        var split;
        split = line.split(': ', 2);
        if (split.length === 2) {
          return fields[split[0]] = split[1];
        }
      });
      if (!(fields.capabilities && fields.encoding)) {
        throw new Error("Expected capabilities and encoding: " + data);
      }
      capabilities = fields.capabilities.split(' ');
      encoding = fields.encoding;
      return {
        capabilities: capabilities,
        encoding: encoding
      };
    };


    /*
    	Parse the Channel information, emit an event on the channel with the data.
     */

    HGCommandServer.prototype._handleServerData = function(data) {
      var body, chan, chanGroups, chanName, currBuffPos, currChan, currGroup, getChanName, group, i, j, len, len1, line, lines;
      lines = [];
      body = "";
      getChanName = function(chan) {
        var chanName;
        chanName = (function() {
          switch (chan) {
            case "o":
              return "output";
            case "r":
              return "result";
            case "e":
              return "error";
            case "d":
              return "debug";
          }
        })();
        return chanName;
      };
      chan = String.fromCharCode(data.readUInt8(0));
      currBuffPos = 0;
      while (currBuffPos < data.length) {
        line = OutputLine.FromBuffer(data, currBuffPos, this.encoding);
        lines.push(line);
        body += line.body + "\n";
        currBuffPos += line.length + 5;
      }
      chanGroups = [];
      currChan = lines[0].channel;
      currGroup = [];
      for (i = 0, len = lines.length; i < len; i++) {
        line = lines[i];
        if (line.channel === currChan) {
          currGroup.push(line);
          continue;
        }
        chanGroups.push(currGroup);
        currGroup = [line];
        currChan = line.channel;
      }
      if (currGroup.length > 0) {
        chanGroups.push(currGroup);
      }
      for (j = 0, len1 = chanGroups.length; j < len1; j++) {
        group = chanGroups[j];
        chanName = getChanName(group[0].channel);
        body = ((function() {
          var k, len2, results;
          results = [];
          for (k = 0, len2 = group.length; k < len2; k++) {
            line = group[k];
            results.push(line.body);
          }
          return results;
        })()).join("\n");
        if (chanName === "error") {
          return this.emit(chanName, new Error(body), group[0], group);
        }
        this.emit(chanName, body, group);
      }
    };

    HGCommandServer.prototype._handleServerError = function(data) {
      return this.emit("error", data);
    };

    HGCommandServer.prototype._handleServerExit = function(code) {
      this.emit("exit", code);
      this.server.removeAllListeners("exit");
      return this.server = void 0;
    };


    /*
    	Send the raw command strings to the cmdserver over `stdin`
     */

    HGCommandServer.prototype._serverSend = function(cmd, args) {
      var argLength, argLengthSize, argParts, cmdLength, toWrite, totalBufferSize;
      if (args == null) {
        args = [];
      }
      if (!this.server) {
        throw new Error("Must start the command server before issuing commands");
      }
      cmdLength = cmd.length + 1;
      argParts = args.join("\0");
      argLength = argParts.length;
      argLengthSize = 4;
      totalBufferSize = cmdLength + argLengthSize + argLength;
      toWrite = new Buffer(totalBufferSize);
      toWrite.write(cmd + "\n", this.encoding);
      toWrite.writeUInt32BE(argLength, cmdLength);
      toWrite.write(argParts, cmdLength + argLengthSize, this.encoding);
      return this.server.stdin.write(toWrite);
    };

    return HGCommandServer;

  })(EventEmitter);

  module.exports = HGCommandServer;

}).call(this);
