// Generated by CoffeeScript 1.10.0
(function() {
  var HGCommandServer, HGRepo, _, fs, fspath, os, spawn, uuid,
    hasProp = {}.hasOwnProperty;

  fs = require("fs");

  os = require("os");

  fspath = require("path");

  spawn = require("child_process").spawn;

  uuid = require("uuid");

  _ = require("lodash");

  HGCommandServer = require("./HGCommandServer");

  HGRepo = (function() {

    /*
    	Create a new repo in a random temp directory.  Useful for no-repo commands like init and clone
    	that require a repo
     */
    HGRepo.MakeTempRepo = function(done) {
      var tmpDir;
      tmpDir = fspath.join(os.tmpDir(), uuid.v1());
      return fs.mkdir(tmpDir, function(err) {
        var initProcess;
        if (err) {
          done(err);
        }
        initProcess = spawn("hg", ["init"], {
          cwd: tmpDir
        });
        return initProcess.on("exit", function(code) {
          if (code !== 0) {
            err = new Error("Non zero status code returned when creating temporary repo: " + code);
            return done(err);
          }
          return done(null, new HGRepo(tmpDir));
        });
      });
    };


    /*
    	Create a new HGRepo with a rootpath defined by the passed in `@path` (defaults to `process.cwd()`)
     */

    function HGRepo(path1) {
      this.path = path1 != null ? path1 : process.cwd();
    }


    /*
    	Initialize a new repository at the provided path.  Due to limitations of the cmdserver,
    	this must be run from an existing repo.
     */

    HGRepo.prototype.init = function(initPath, done) {
      var serverCmd;
      serverCmd = function(server) {
        return server.runcommand("init", initPath);
      };
      return this._runCommandGetOutput(this.path, serverCmd, done);
    };


    /*
    	Add files to a repository.
     */

    HGRepo.prototype.add = function(paths, done) {
      return this.runCommand("add", paths, done);
    };


    /*
    	Commit changes to a repository
     */

    HGRepo.prototype.commit = function(paths, opts, done) {
      if (_.isFunction(paths)) {
        done = paths;
        paths = [];
      } else if (_.isFunction(opts)) {
        done = opts;
        if (_.isObject(paths)) {
          opts = paths;
          paths = [];
        }
      }
      return this.runCommand(["commit"].concat(paths), opts, done);
    };


    /*
    	Clone a repository.  Due to limitations of the cmdserver, this must be run from an
    	existing location.
     */

    HGRepo.prototype.clone = function(from, to, opts, done) {
      return this.runCommand(["clone", from, to], opts, done);
    };


    /*
    	Get a summary of the current repository path.
     */

    HGRepo.prototype.summary = function(opts, done) {
      return this.runCommand("summary", opts, done);
    };


    /*
    	Get a log of commits for this repository.
    
    	`opts` is optional and can be either an object or array of arguments.
     */

    HGRepo.prototype.log = function(opts, done) {
      return this.runCommand("log", opts, done);
    };


    /*
    	Pull changes from another repository.
     */

    HGRepo.prototype.pull = function(from, opts, done) {
      return this.runCommand(["pull", from], opts, done);
    };


    /*
    	Update to the latest changes in a repository.
     */

    HGRepo.prototype.update = function(opts, done) {
      return this.runCommand("update", opts, done);
    };


    /*
    	Push changes to another repository
     */

    HGRepo.prototype.push = function(to, opts, done) {
      return this.runCommand(["push", to], opts, done);
    };


    /*
    	Merge changes from another repository
     */

    HGRepo.prototype.merge = function(opts, done) {
      return this.runCommand("merge", opts, done);
    };


    /*
    	Resolve conflicts in a repository.
     */

    HGRepo.prototype.resolve = function(opts, done) {
      return this.runCommand("resolve", opts, done);
    };


    /*
    	Create tags in repo.
     */

    HGRepo.prototype.tag = function(tagname, opts, done) {
      return this.runCommand(["tag", tagname], opts, done);
    };


    /*
    	Retrieve repo tags.
     */

    HGRepo.prototype.tags = function(opts, done) {
      return this.runCommand("tags", opts, done);
    };


    /*
    	Repo status.
     */

    HGRepo.prototype.status = function(opts, done) {
      return this.runCommand("status", opts, done);
    };


    /*
    	Repo branches.
     */

    HGRepo.prototype.branches = function(opts, done) {
      return this.runCommand("branches", opts, done);
    };


    /*
    	Repo heads list
     */

    HGRepo.prototype.heads = function(opts, done) {
      return this.runCommand("heads", opts, done);
    };


    /*
    	Diff
     */

    HGRepo.prototype.diff = function(opts, done) {
      return this.runCommand("diff", opts, done);
    };


    /*
    	Version of hg process
     */

    HGRepo.prototype.version = function(done) {
      return this.runCommand("version", done);
    };


    /*
    	Remove files from a repository.
     */

    HGRepo.prototype.remove = function(paths, done) {
      return this.runCommand("remove", paths, done);
    };


    /*
    	Execute server command
    
    	@param arg [String, Array] command to execute.
    	@param opts [Object, Array, String, Function] optional arguments to append
    	       to command args. If opts is a function it is treated as the callback function.
    	@param done [Function] callback when command completes
     */

    HGRepo.prototype.runCommand = function(args, opts, done) {
      var serverCmd;
      if (_.isFunction(opts)) {
        done = opts;
        opts = [];
      }
      if (_.isString(args)) {
        args = [args];
      }
      args = args.concat(this._parseOptions(opts));
      serverCmd = function(server) {
        return server.runcommand.apply(server, args);
      };
      return this._runCommandGetOutput(this.path, serverCmd, done);
    };


    /*
    	Parse an object into an array of command line arguments
     */

    HGRepo.prototype._parseOptions = function(opts) {
      var currKey, key, newOpts, pushVal, val;
      if (_.isArray(opts)) {
        return opts;
      }
      if (_.isString(opts)) {
        return [opts];
      }
      newOpts = [];
      currKey = "";
      pushVal = function(v) {
        newOpts.push(currKey);
        if (v) {
          return newOpts.push(v);
        }
      };
      for (key in opts) {
        if (!hasProp.call(opts, key)) continue;
        val = opts[key];
        currKey = key;
        if (_.isArray(val)) {
          _.each(val, pushVal);
        } else {
          pushVal(val);
        }
      }
      return newOpts;
    };


    /*
    	Start a command server and return it for use
     */

    HGRepo.prototype._startServer = function(path, done) {
      var server;
      server = new HGCommandServer();
      return server.start(path, function(err) {
        if (err) {
          return done(err);
        }
        return done(null, server);
      });
    };


    /*
    	Convenience wrapper for starting a command server and executing a command
     */

    HGRepo.prototype._runCommandGetOutput = function(path, serverAction, done) {
      return this._startServer(path, function(err, server) {
        var allOutput, cleanUp;
        if (err) {
          return done(err);
        }
        cleanUp = function() {
          server.removeAllListeners("output");
          return server.removeAllListeners("error");
        };
        allOutput = [];
        server.on("output", function(body, lines) {
          return allOutput = allOutput.concat(lines);
        });
        server.on("error", function(err, line) {
          var ref;
          if ((line != null ? (ref = line.body) != null ? ref.slice(0, 7) : void 0 : void 0) === "warning") {
            return allOutput.push(line);
          }
          cleanUp();
          return done(err);
        });
        server.once("result", function(body, lines) {
          if (lines.length > 0) {
            allOutput = allOutput.concat(lines);
          }
          return server.stop();
        });
        server.once("exit", function(code) {
          cleanUp();
          return done(null, allOutput, server);
        });
        return serverAction(server);
      });
    };

    return HGRepo;

  })();

  module.exports = HGRepo;

}).call(this);
