// Generated by CoffeeScript 1.10.0
(function() {
  var Parsers,
    slice = [].slice;

  Parsers = (function() {

    /*
    Parse command output as raw text content.
    
        repo.branches (err, out) ->
           console.log Parsers.text(out)
     */
    Parsers.text = function(out) {
      var o;
      return ((function() {
        var i, len, results;
        results = [];
        for (i = 0, len = out.length; i < len; i++) {
          o = out[i];
          if (o.channel === 'o') {
            results.push(o.body);
          }
        }
        return results;
      })()).join('');
    };


    /*
    Parse command output as json content. This is useful when JSON template
    is passed as an option to the command (i.e. `-Tjson`).
    
        repo.branches {"--template":"json"}, (err, out) ->
            branches = Parsers.json out
            branches.forEach (b) ->
               console.log "#{b.branch} #{b.active}"
     */

    Parsers.json = function(out) {
      return JSON.parse(Parsers.text(out));
    };


    /*
    Parse version from `version` command. Returns a string version number.
     */

    Parsers.version = function(out) {
      var version, versionRegEx;
      versionRegEx = new RegExp(".*version (.*)\\)");
      version = versionRegEx.exec(out[0].body);
      if (!version) {
        throw new Error("Unable to parse version data");
      }
      return version[1];
    };

    function Parsers(hgversion) {
      this.hgversion = hgversion;
    }


    /*
    Parse `tags` command text response. Returns an object with tag names as
    object keys. Key value is a 2 element array of revision number and revision
    hash.
    
        tags = {"<tag name>":["<rev number>":"<hash>"]}
     */

    Parsers.prototype.tags = function(out) {
      var hver, lver, name, namerec, nl, ref, ref1, rest, tags, verrec;
      rest = out;
      tags = {};
      while (rest[0].channel !== 'r') {
        ref = rest, namerec = ref[0], verrec = ref[1], nl = ref[2], rest = 4 <= ref.length ? slice.call(ref, 3) : [];
        name = namerec.body;
        ref1 = verrec.body.split(":"), lver = ref1[0], hver = ref1[1];
        tags[name] = [lver.trim(), hver.trim()];
      }
      return tags;
    };


    /*
    Parse `status` command text response. Returns an object with file names as
    object keys. Key value is the status id.
    
        status = {"<file name>":"<status>"}
     */

    Parsers.prototype.status = function(out) {
      var file, filerec, ref, rest, state, staterec, states;
      rest = out;
      states = {};
      while (rest[0].channel !== 'r') {
        ref = rest, staterec = ref[0], filerec = ref[1], rest = 3 <= ref.length ? slice.call(ref, 2) : [];
        state = staterec.body.trim();
        file = filerec.body.trim();
        states[file] = state;
      }
      return states;
    };

    return Parsers;

  })();

  module.exports = Parsers;

}).call(this);
